import { NativeEventEmitter, NativeModules, Platform } from 'react-native';
const {
  IONPortalPubSub,
  IONPortalsReactNative,
  IONPortalsWebVitals
} = NativeModules;
export { default as PortalView } from './PortalView';

/**
 * The data that is received from a subscription event.
 */

const PortalsPubSub = new NativeEventEmitter(IONPortalPubSub);
const subscriptionMap = new Map();

/**
 * Subscribes to messages for a topic
 *
 * @param topic The topic to subscribe to
 * @param onMessageReceived The callback to invoke when a message is received
 * @returns A Promise<number> containing the unique subscription reference. This will need to be stored for calling {@link unsubscribe}.
 */
export const subscribe = async (topic, onMessageReceived) => {
  const subscriptionRef = await IONPortalPubSub.subscribe(topic);
  const subscriber = PortalsPubSub.addListener('PortalsSubscription', message => {
    if (message.subscriptionRef === subscriptionRef) {
      onMessageReceived(message);
    }
  });
  subscriptionMap.set(subscriptionRef, subscriber);
  return subscriptionRef;
};
const webVitalsMap = new Map();
const WebVitals = new NativeEventEmitter(IONPortalsWebVitals);
export const onFirstContentfulPaint = async (portalName, callback) => {
  const listener = WebVitals.addListener('vitals:fcp', event => {
    if (event.portalName === portalName) {
      callback(event.duration);
    }
  });
  await IONPortalsWebVitals.registerOnFirstContentfulPaint(portalName);
  webVitalsMap.set(`${portalName}-vitals:fcp`, listener);
};
export const onFirstInputDelay = async (portalName, callback) => {
  if (Platform.OS === 'android') {
    const listener = WebVitals.addListener('vitals:fid', event => {
      if (event.portalName === portalName) {
        callback(event.duration);
      }
    });
    await IONPortalsWebVitals.registerOnFirstInputDelay(portalName);
    webVitalsMap.set(`${portalName}-vitals:fcp`, listener);
  }
};
export const onTimeToFirstByte = async (portalName, callback) => {
  if (Platform.OS === 'android') {
    const listener = WebVitals.addListener('vitals:ttfb', event => {
      if (event.portalName === portalName) {
        callback(event.duration);
      }
    });
    await IONPortalsWebVitals.registerOnTimeToFirstByte(portalName);
    webVitalsMap.set(`${portalName}-vitals:ttfb`, listener);
  }
};
export const registerWebVitals = async (portalName, firstContentfulPaint, firstInputDelay, timeToFirstByte) => {
  onFirstContentfulPaint(portalName, firstContentfulPaint);
  onFirstInputDelay(portalName, firstInputDelay);
  onTimeToFirstByte(portalName, timeToFirstByte);
};

/**
 * Unsubscribes from events for the provided topic and subscription reference
 *
 * @param topic The topic to unsubscribe from
 * @param subRef The unique subscription reference received when initially calling {@link subscribe}
 */
export const unsubscribe = (topic, subRef) => {
  IONPortalPubSub.unsubscribe(topic, subRef);
  const subscription = subscriptionMap.get(subRef);
  if (subscription !== undefined) {
    subscription.remove();
    subscriptionMap.delete(subRef);
  }
};

/**
 * Publishes a message to the provided topic
 *
 * @param topic The topic to publish the message to
 * @param data The data to publish to subscribers
 */
export const publish = (topic, data) => {
  const msg = {
    message: data
  };
  IONPortalPubSub.publish(topic, msg);
};

/**
 * Validates that a valid registration key has been procured from http://ionic.io/register-portals
 * @param key The registration key
 * @returns Promise<void>
 */
export const register = async key => {
  return IONPortalsReactNative.register(key);
};

/**
 * The configuration of a web application to be embedded in a React Native application.
 */

/**
 * A subset of {@link Portal} properties needed for rendering a Portal. `initialContext` can be used to override
 * any initialContext defined in the original {@link Portal} definition.
 */

/**
 * Props needed for rendering a {@link Portal}
 */

/**
 * Adds a Portal to an internal registry. Must be called before attempting to render a {@link PortalView}.
 *
 * @param portal The portal to add to the internal registry.
 * @returns Promise containing the Portal that was added to the registry.
 */
export const addPortal = async portal => {
  return IONPortalsReactNative.addPortal(portal);
};

/**
 * Adds all portals to an internal registry. This or {@link addPortal} must be called before attempting to render a {@link PortalView}
 *
 * @param portals The portals to add to the internal registry.
 * @returns Promise containing the Portals that were added to the registry.
 */
export const addPortals = async portals => {
  return IONPortalsReactNative.addPortals(portals);
};

/**
 * Gets a {@link Portal} previously registered via {@link addPortal} or {@link addPortals}.
 *
 * @param name The portal name to retrieve from the internal registry.
 * @returns Promise containing the registered {@link Portal}. If the {@link Portal} was not registered, the Promise will fail.
 */
export const getPortal = async name => {
  return IONPortalsReactNative.getPortal(name);
};

/** Data needed to register a live update to be managed */

/** Used for communicating sync results of multiple live updates */

/**
 * Configures LiveUpdates to cyrptographically verify the contents of the downloaded bundles.
 * This method must be called before any LiveUpdates are registered otherwise they will no longer be tracked.
 *
 * @param pathToKey The *relative* path to the public key for verification.
 * This path should be the same relatibe to the main application bundle on iOS and the assets directory on Android.
 * @returns Promise<void>
 */
export const enableSecureLiveUpdates = async pathToKey => {
  return IONPortalsReactNative.enableSecureLiveUpdates(pathToKey);
};

/**
 * Syncs a single live update.
 *
 * @param appId The AppFlow application ID to sync.
 * @returns A Promise<LiveUpdate>. A failure should result in a {@link LiveUpdateError}.
 */
export const syncOne = async appId => {
  return IONPortalsReactNative.syncOne(appId);
};

/**
 * Syncs many live updates.
 *
 * @param appIds The AppFlow application IDs to sync.
 * @returns Promise<SyncResults>
 */
export const syncSome = async appIds => {
  return IONPortalsReactNative.syncSome(appIds);
};

/**
 * Syncs all registered LiveUpdates
 * @returns Promise<SyncResults>
 */
export const syncAll = async () => {
  return IONPortalsReactNative.syncAll();
};
//# sourceMappingURL=index.js.map